﻿using IdeaBlade.EntityModel.Server;

namespace Server {

  /// <summary>
  /// Custom implementations of EntityServerQueryInterceptor and EntityServerSaveInterceptor.
  /// Methods in these classes will be called by DevForce (on the server) during any fetch or save.
  /// Here we've disallowed all queries and saves except from authenticated users.
  /// </summary>
  public class EntityServerQueryManager : EntityServerQueryInterceptor {

    protected override bool DefaultAuthorization {
      get {
        return false;
      }
    }

    protected override bool AuthorizeQuery() {
      return Principal.Identity.IsAuthenticated;
    }

  }

  public class EntityServerSaveManager : EntityServerSaveInterceptor {

    protected override bool DefaultAuthorization {
      get {
        return false;
      }
    }

    protected override bool AuthorizeSave() {
      return Principal.Identity.IsAuthenticated;
    }

  }

}