﻿Imports System.Security.Principal

Imports IdeaBlade.Core
Imports IdeaBlade.EntityModel

Namespace Server

	''' Server-side login manager using Windows Authentication. 
	'''  - Remember to place Server.dll in exe directory of client if 2-tier or on server in n-tier.
	'''  
	'''  To use Windows authentication with DevForce: 
	'''    We pass credentials along the WCF channel from the client to the EntityService, 
	'''    which requires defining a system.ServiceModel section in the configuration files
	'''    on both client and server and setting binding information.  Notice the binding
	'''    called "wsBindingCustom" in both the client and server config file samples - this
	'''    binding passes Windows credentials to the service.  Other binding configurations
	'''    are also possible.
	'''    
	'''  The client application logs in with "null credentials", and here in the
	'''  LoginManager we'll grab the Thread.CurrentPrincipal and use that as the authenticated
	'''  user.  We need to return an IPrincipal here; DevForce uses whatever is returned here
	'''  as the "logged in" user on the client, and also for all subsequent requests from the client
	'''  to the BOS.
  Public Class LoginManager
	  Implements IEntityLoginManager

	Public Sub New()
	End Sub

    Public Function Login(ByVal credential As ILoginCredential, ByVal entityManager As EntityManager) As IPrincipal Implements IEntityLoginManager.Login

      If credential Is Nothing Then

        ' if running n-tier the user's Windows credentials will be passed and available on the thread:
        Dim principal = System.Threading.Thread.CurrentPrincipal
        If principal.Identity.IsAuthenticated Then
          WriteMsg(String.Format("principal is {0}", principal.Identity.Name))
          ' Note that we can't return the WindowsPrincipal/WindowsIdentity back to the client, so
          ' create a new one, here we use the DevForce UserBase class, but you can create your own too. 
          Return CreateUser(principal)
        Else
          ' This is here for 2-tier testing, since you'll probably want to throw an error or do
          ' some custom logic in n-tier.  When not running n-tier, the thread principal has not been set 
          ' but you can obtain the Windows Identity and use that.
          Dim wi = WindowsIdentity.GetCurrent()
          WriteMsg(String.Format("wi is {0}", wi.Name))
          Dim wp As New WindowsPrincipal(wi)
          Return CreateUser(wp)
        End If
      End If

      Throw New NotImplementedException("Only Windows authentication is supported here")
    End Function

    Public Sub Logout(ByVal principal As IPrincipal, ByVal entityManager As EntityManager) Implements IEntityLoginManager.Logout
    End Sub

	Private Sub WriteMsg(ByVal msg As String)
	  Console.WriteLine(msg)
	  TraceFns.WriteLine(msg)
	End Sub

	Private Function CreateUser(ByVal currentUser As IPrincipal) As IPrincipal
	  ' The UserBase and UserIdentity classes are custom IPrincipal/IIdentity implementations
	  ' within DevForce.  

	  Dim identity As New UserIdentity(currentUser.Identity.Name, currentUser.Identity.AuthenticationType, currentUser.Identity.IsAuthenticated)
	  ' You can determine what roles to set, if any.
	  Dim roles() As String = { }
	  Dim user As New UserBase(identity, roles)
	  Return user
	End Function

  End Class
End Namespace
